from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.urls import reverse

from django.contrib.auth import login, authenticate
from django.contrib.auth.models import auth, User
from . import forms
from . import models
import datetime

# Create your views here.
def index(request):
    # get a count of all current users and orders from db
    orderCount = models.PizzaOrder.objects.all().count()
    userCount = User.objects.all().count()

    return render(request, "index.html", {"orderCount": orderCount, "userCount": userCount})

def me(request):
    # ensure logged in (could also use decorator but this works well with messages)
    if (not request.user.is_authenticated):
        # display a message to the user (Django messaging system)
        messages.warning(request, "You are not logged in.")
        return redirect(f"login?next={request.path}")
    
    # get all orders for the logged in user
    orders = models.PizzaOrder.objects.filter(orderedBy_id=request.user.id)

    paymentDue = request.session.get("order_id")

    return render(request, "me.html", {'orders': orders, 'paymentDue': paymentDue})

def order(request):
    if (not request.user.is_authenticated):
        messages.warning(request, "Please log in first.")
        return redirect(f"login?next={request.path}")

    toppings = models.Topping.objects.all()
    
    if request.method == "POST":
        form = forms.PizzaOrderForm(request.POST)
        if form.is_valid():
            instance = form.save(commit=False)
            
            # Modify instance fields before saving
            instance.orderedBy = request.user

            instance.save()

            # add toppings from custom checkboxes
            toppingIds = request.POST.getlist("topping")

            selectedToppings = models.Topping.objects.filter(id__in=toppingIds)
            
            instance.toppings.set(selectedToppings)

            # Save the instance to the database
            instance.save()

            # set order id in the session
            request.session["order_id"] = instance.orderId

            messages.info(request, "Order saved, enter delivery details")
            return redirect("payment")
        else:
            return render(request, "order.html", {"form": form, "toppings": toppings})

    else:
        form = forms.PizzaOrderForm()

        return render(request, "order.html", {"form": form, "toppings": toppings})

def payment(request):
    if (not request.user.is_authenticated):
        messages.warning(request, "Please log in first.")
        return redirect(f"login?next={request.path}")
    
    # feature to also pay for any order from the user not just the current one
    # this uses the for url param
    orderId = request.GET.get("for")

    fromParams = True

    if orderId is None:    
        orderId = request.session.get("order_id")
        fromParams = False
        if not orderId:
            messages.warning(request, "State Error: Please re-create your order.")
            return redirect("order")

    order = get_object_or_404(models.PizzaOrder, orderId=orderId)

    # check to ensure the user who owns order is the one logged in
    if order.orderedBy != request.user:
        messages.warning(request, "You don't have permission to edit this order.")
        return redirect("me")
    
    # ensure order hasn't already been paid for
    if order.paymentComplete:
        messages.warning(request, "This order has already been paid for! Unless you want to pay again?...")
        return redirect("me")

    # year info for card expiration
    currentYear = datetime.date.today().year

    if request.method == "POST":
        form = forms.DeliveryInfoForm(request.POST)#
        if form.is_valid():
            instance = form.save(commit=False)
            
            # Modify instance fields before saving
            instance.order = order

            cardExpiryMonth = request.POST.get("cardExpiryMonth")
            cardExpiryYear = request.POST.get("cardExpiryYear")

            # create a new time instance
            cardExpiry = datetime.date(int(cardExpiryYear), int(cardExpiryMonth), 1)

            instance.cardExpiry = cardExpiry

            # Save the instance to the database
            instance.save()

            # mark payment as complete in order table
            order.paymentComplete = True
            order.save()

            # if from params (?for=) add to session for confirm page to work
            if fromParams:
                request.session["order_id"] = order.orderId
            
            return redirect("confirmation")
        else:
            return render(request, "payment.html", {"form": form, "order": order, "months": range(1, 13), "years": range(currentYear, currentYear + 30)})
    else:
        form = forms.DeliveryInfoForm()

        return render(request, "payment.html", {"form": form, "order": order, "months": range(1, 13), "years": range(currentYear, currentYear + 30)})

def cancel(request):
    if (not request.user.is_authenticated):
        messages.warning(request, "Please log in first.")
        return redirect(f"login?next={request.path}")
    
    orderId = request.POST.get("for")

    if orderId is None:    
        return redirect("me")

    order = get_object_or_404(models.PizzaOrder, orderId=orderId)

    # check to ensure the user who owns order is the one logged in
    if order.orderedBy != request.user:
        messages.warning(request, "You don't have permission to edit this order.")
        return redirect("me")
    
    # ensure order hasn't already been paid for
    if order.paymentComplete:
        messages.warning(request, "Cannot delete an order which has been paid for.")
        return redirect("me")
    
    if request.method == "POST":
        models.PizzaOrder.delete(order)

        if request.session.get("order_id"):
            del request.session["order_id"]

        messages.success(request, f"Order (ID: {orderId}) deleted.")
        return redirect("me")

    else:
        return redirect("me")
    
def confirm(request):
    if (not request.user.is_authenticated):
        messages.warning(request, "Please log in first.")
        return redirect(f"login?next={request.path}")
    
    orderId = request.session.get("order_id")
    if not orderId:
        messages.warning(request, "State Error: Please re-create your order.")
        return redirect("order")

    order = get_object_or_404(models.PizzaOrder, orderId=orderId)


    # check to ensure the user who owns order is the one logged in
    if order.orderedBy != request.user:
        messages.warning(request, "You don't have permission to view this order.")
        return redirect("me")
    
    # ensure order hasn't already been paid for
    if not order.paymentComplete:
        messages.warning(request, "This order hasn't been paid for yet.")
        return redirect(f"/payment?for={order.orderId}")
    
    if request.session.get("order_id"):
        del request.session["order_id"]

    # 30 mins time for delivery
    estimatedTime = order.delivery.paid_at + datetime.timedelta(minutes=30)

    return render(request, "confirm.html", {"order": order, "estimatedDeliveryTime": estimatedTime})

def registration(request):
    # if sending data
    if request.method == "POST":
        form = forms.AccountRegisterForm(request.POST)
        if form.is_valid():
            password = form.cleaned_data["password1"]
            newUser = form.save()

            authenticated_user = authenticate(username=newUser.username, password=password)

            if authenticated_user is not None:
                login(request, authenticated_user)
                messages.success(request, "Account created successfully!")
                return redirect("me")
            
            else:
                messages.warning(request, "Account created but failed to login, login manually!")
                return redirect("login")

        else:
            messages.warning(request, "Validation Failed")
            return render(request, 'register.html', {'form': form})

    else:
        # else just show form
        form = forms.AccountRegisterForm()
        return render(request, "register.html", {'form': form})

def logout(request):
    auth.logout(request)

    messages.success(request, "You have successfully logged out.")

    return redirect("index")